#ifndef_COMLIB_H
#define _COMLIB_H

/*
 * function number
 */
#define COMINIT         0x00
#define COMSEND         0x01
#define COMRECV         0x02
#define COMSTAT         0x03
#define COMACTIVE       0x04
#define COMSETDTR       0x06
#define COMPURGERXBUF   0x0A
#define COMPEEK         0x0C
#define COMSETFLOW      0x0F
#define COMRXBLK        0x18
#define COMTXBLK        0x19
#define COMSETBREAK     0x1A
#define COMINFO         0x1B

#define COMGETBAUD      0x2E
#define COMSETBAUD      0x2F
#define COMBUFSTAT      0x30
#define COMSETPORT      0x31
#define COMSETVECTOR    0x32
#define COMSETACTIVE    0x33
#define COMSETMCR       0x34
#define COMGETPARAM     0x35
#define COMGETMCR       0x36
#define COMGETFC        0x37
#define COMGETSETS      0x38
#define COMGETSETINFO   0x39
#define COMGETPORTS     0x3A
#define COMDETECTPORT   0x3B
#define COMGETPORTINFO  0x3C
#define COMRXOFF        0x3D
#define COMRXON         0x3E
#define COMSETTIMEOUT   0x3F
#define COMGETTIMEOUT   0x40

/*
 * line control
 */
#define B600            600
#define B1200           1200
#define B9600           9600
#define B38400          38400
#define B56000          56000
#define B115200         115200
#define B230400         230400
#define B460800         460800
#define CS5             5
#define CS6             6
#define CS7             7
#define CS8             8
#define ST1             1
#define ST2             2
#define PNONE           0
#define PODD            1
#define PEVEN           2
#ifndef ACTIVE
#define ACTIVE          1
#endif  ACTIVE
#ifndef INACTIVE
#define INACTIVE        0
#endif  INACTIVE
#ifndef XON
#define XON             17
#endif  XON
#ifndef XOFF
#define XOFF            19
#endif  XOFF

/*
 * status for serial port (function 1,2,3)
 */
#define S_DCTS		0x0001
#define S_DDTR		0x0002
#define S_DDCD		0x0004
#define S_DUMMY_DCD	0x0008		// always 1
#define S_CTS           0x0010
#define S_DSR           0x0020
#define S_RI		0x0040
#define S_DCD           0x0080
#define S_RDA           0x0100
#define S_OVRN		0x0200
#define S_RVSD1		0x0400
#define S_RVSD2		0x0800
#define S_RVSD3		0x1000
#define S_THRE          0x2000
#define S_TSRE          0x4000
#define S_TIMEOUT       0x8000


/*
 *  modem control
 */
#define M_DTR           0x01
#define M_RTS           0x02
#define M_OUT1          0x04
#define M_OUT2          0x08
#define M_LOOP          0x10


/*
 * flow control
 */
#define F_XONXOFF       0x09
#define F_RTSCTS        0x02
#define F_DTRDSR        0x04

/*
 * fossil driver information
 */
struct FOSSIL_INFO {
   unsigned int  size;
   unsigned char majver;
   unsigned char minver;
   char          far *ident;
   unsigned int  ibufr;
   unsigned int  ifree;
   unsigned int  obufr;
   unsigned int  ofree;
   unsigned char swidth;
   unsigned char sheight;
   unsigned char baud;
};

/*
 * information about one pccom set
 */
typedef struct {
           unsigned char irq;
           unsigned int  portno;
           unsigned int  groupno;
	   unsigned int  whichgroup;

           struct {
              unsigned int  base;
              unsigned int  vector;
	      unsigned char active;
              unsigned char portno;
           } group[8];
        } BiosComSet_t;

/*
 * information about pccom port
 */
typedef struct {
           unsigned int  addr;
           unsigned int  baud;
           unsigned char lcr;
           unsigned char mcr;
           unsigned char fc;
           unsigned int  rx_qsz;
	   unsigned int  rx_qfree;
           unsigned char rx_enable;
           unsigned char tx_enable;
        } BiosComPort_t;

/*
 * function prototype
 */
int  cominit(int portno, long baud, int db, int parity, int sb);
int  comtxch(int portno, char c);
int  comrxch(int portno, char *c);
int  comstat(int portno);
int  comactive(int portno);
void comsetDTR(int portno, int dtr);
void compurgerxbuf(int portno);
int  compeek(int portno);
void comsetflow(int portno, int flow);
int  comtxblk(int portno, void *ptr, int size);
int  comrxblk(int portno, void *ptr, int size);
void combreak(int port, int brk);
int  cominstalled(int portno);
void comsetRTS(int portno, int rts);
int  comgetbufstat(int portno);
int  comgetDCD(int portno);
int  comgetDSR(int portno);
int  comgetCTS(int portno);
int  comgetflow(int portno);
void comgetinfo (int portno, long *baud, int *db, int *parity, int *sb);
int  comtxs(int portno, char *s);
int  comrxs(int portno, char *s);

int           combufstat(int portno);
int           comtxbufstat(int portno);
void          comsetport(int portno, int addr);
void          comsetvector(int portno, int addr);
void          comsetactive(int portno,char active);
void          comsetMCR(int portno, int mcr);
char          comgetparam(int portno);
char          comgetMCR(int portno);
char          comgetFC(int portno);
int           comgetsets(int portno);
BiosComSet_t  *comgetsetinfo(int portno);
int           comgetports(int portno);
int           comdetectport(int portno);
BiosComPort_t *comgetportinfo(int portno);
void          comrxoff(int portno);
void          comrxon(int portno);
void          comsettimeout(int portno, unsigned int timeout);
unsigned int  comgettimeout(int portno);
int           comsetbaud(int portno, unsigned long baud);

#endif _COMLIB_H

