/*
 * PCCOM setup program    By Decision-Computer    Aug. 93'
 */
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>
#include <string.h>

#include "comlib.h"

/*
 * type defnitions used in setup.c
 */
typedef struct {
	   unsigned int  portno;	/* bios port no */
	   unsigned int  addr;		/* uart address of this port */
	   unsigned char irq;		/* irq by which this port is trigger */
	   unsigned int  vector;	/* address of vector about this port */
	   unsigned char active;	/* active status in vector */
	   unsigned int  baud;		/* baud rate */
	   int  	 parity;	/* parity */
	   int  	 db;		/* data bits */
	   int  	 sb;		/* stop bits */
	   unsigned char mc;		/* modem control value */
	   unsigned char fc;		/* flow control value */
	} PCCOM_t;


/*
 * Constants used in setup.c
 */
const char *TITLE =
"\n\n\n                    PCCOM SETUP PROGRAM V2.0\n\n\
	 <<< DECISION-COMPUTER INTERNATIONAL CO.,LTD. >>>\n\n";

const char *USAGE =
" Usage : [\/v,]portno[,baud,parity,data,stop][,MC=Ms][,FC=Fs]\n\
	    \/v	     = verbose\n\
	    baud     = 300, 600, 1200, 2400, 4800, 9600, 19200, 38400\n\
	    parity   = N(one), E(ven), O(dd)\n\
	    data     = 5, 6, 7, 8\n\
	    stop     = 1, 2\n\
	    M(modem) = DTR, RTS, OUT1, OUT2, NONE\n\
	    F(flow)  = XON, DTR, RTS, NONE\n\n";

const char *EXAMPLE =
" Example : SETUP 4\n\
 Example : SETUP 5,9600,N,8,1,MC=DTR+RTS,FC=XON\n\
 Example : SETUP 6,19200,E,7,2\n\
 Example : SETUP 7,FC=DTR+RTS\n";


/*
 * show usage if argument to setup is bad
 */
void usage(void)
{
   puts(TITLE);
   puts(USAGE);
   puts(EXAMPLE);

   exit(1);
}

void setup_err(int id)
{
   switch (id) {
      case 0 :	/* bad modem control value */
	 printf("Error : Bad Modem Control Value\n");
	 printf("MC=DTR, RTS, OUT1, OUT2, or NONE\n");
	 break;
      case 1 :
	 printf("Error : Bad Flow Control Value\n");
	 printf("MC=RTS, XON, NONE\n");
	 break;
   }

   exit(2);
}

void PCCOMGetMC(unsigned char *mc, char *s)
{
   *mc = 0x00;
   while (*s) {
      if (strnicmp(s, "DTR", 3) == 0)
	 *mc |= M_DTR,  s += 3;
      else if (strnicmp(s, "RTS", 3) == 0)
	 *mc |= M_RTS,  s += 3;
      else if (strnicmp(s, "OUT1", 4) == 0)
	 *mc |= M_OUT1, s += 4 ;
      else if (strnicmp(s, "OUT2", 4) == 0)
	 *mc |= M_OUT2, s += 4;
      else if (strnicmp(s, "NONE", 4) == 0)
	 *mc = 0, s += 4;
      else if (strnicmp(s, "+", 1))
	 s++;
      else
	 setup_err(0);
   }
}

void PCCOMGetFC(unsigned char *fc, char *s)
{
   *fc = 0x00;
   while (*s) {
      if (strnicmp(s, "XON", 3) == 0)
	 *fc |= F_XONXOFF,  s += 3;
      else if (strnicmp(s, "RTS", 3) == 0)
	 *fc |= F_RTSCTS,  s += 3;
      else if (strnicmp(s, "DTR", 3) == 0)
	 *fc |= F_DTRDSR,  s += 3;
      else if (strnicmp(s, "NONE", 4) == 0)
	 *fc = 0, s += 4;
      else if (strnicmp(s, "+", 1) == 0)
	 s += 1;
      else
	 setup_err(1);
   }
}

void PCCOMGetPort(PCCOM_t *PCCOM)
{
   BiosComSet_t  *sinfo;
   BiosComPort_t *pinfo;

   sinfo = comgetsetinfo(PCCOM->portno);
   PCCOM->irq    = sinfo->irq;
   PCCOM->vector = sinfo->group[sinfo->whichgroup].vector;
   PCCOM->active = sinfo->group[sinfo->whichgroup].active;

   pinfo = comgetportinfo(PCCOM->portno);
   PCCOM->addr = pinfo->addr;
   PCCOM->mc   = pinfo->mcr;
   PCCOM->fc   = pinfo->fc;

   comgetinfo(PCCOM->portno, &PCCOM->baud, &PCCOM->db, &PCCOM->parity, &PCCOM->sb);
}

void PCCOMSetPort(PCCOM_t *PCCOM)
{
   comsetMCR(PCCOM->portno, PCCOM->mc);
   comsetflow(PCCOM->portno, PCCOM->fc);

   cominit(PCCOM->portno, PCCOM->baud, PCCOM->db, PCCOM->parity, PCCOM->sb);
}

void PCCOMShowPort(PCCOM_t *PCCOM)
{
   int flag;

   printf("\n");

   printf("       VECTOR : %X\n", PCCOM->vector);

   printf("       ACTIVE : ");
   if (PCCOM->active)
      printf("HIGH\n");
   else
      printf("LOW\n");

   printf(" PORT ADDRESS : %3X", PCCOM->addr);
   if (!comdetectport(PCCOM->portno)) printf("(????)");
   printf("\n");

   printf("    BAUD RATE : %u\n", PCCOM->baud);

   printf("       PARITY : ");
   switch (PCCOM->parity) {
      case PNONE :
	 printf("NONE\n");
	 break;
      case PODD :
	 printf("ODD\n");
	 break;
      case PEVEN :
	 printf("EVEN\n");
	 break;
   }

   printf("    DATA BITS : %d\n", PCCOM->db);

   printf("    STOP BITS : %d\n", PCCOM->sb);

   flag = 0;
   printf("MODEM CONTROL : ");
   if (PCCOM->mc & M_DTR) {
      flag = 1;
      printf("DTR");
   }
   if (PCCOM->mc & M_RTS) {
       if (flag > 0) printf("+");
       flag = 1;
       printf("RTS");
   }
   if (PCCOM->mc & M_OUT1) {
       if (flag > 0) printf("+");
       flag = 1;
       printf("OUT1");
   }
   if (PCCOM->mc & M_OUT1) {
       if (flag > 0) printf("+");
       flag = 1;
       printf("OUT2");
   }
   if (!flag) printf("NONE");
   printf("\n");

   flag = 0;
   printf(" FlOW CONTROL : ");
   if (PCCOM->fc & F_XONXOFF) {
      flag = 1;
      printf("XON/XOFF");
   }
   if (PCCOM->fc & F_DTRDSR) {
       if (flag > 0) printf("+");
       flag = 1;
       printf("DTR/DSR");
   }
   if (PCCOM->fc & F_RTSCTS) {
       if (flag > 0) printf("+");
       flag = 1;
       printf("RTS/CTS");
   }
   if (!flag) printf("NONE");
   printf("\n\n");
}

void main(int argc, char *argv[])
{
   PCCOM_t	 PCCOM;
   char          *s;
   unsigned char verbose = 0, just_show = 0;
   int		 i;

   if (argc == 1)
      usage();
   else {
      /*
       * extract port no
       */
      s = strtok(argv[1], ",");
      if (strnicmp(s, "/v", 2) == 0) {
	 verbose = 1;
	 s = strtok(NULL, ",");
      }
      PCCOM.portno = strtoul(s, NULL, 10);

      if (!cominstalled(PCCOM.portno)) {
	 printf("PCCOM is not installed\n");
	 exit(1);
      }

      /*
       * Get all information about specified port
       */
      PCCOMGetPort(&PCCOM);

      /*
       * determine if there are other optional arguments
       */
      if ((s = strtok(NULL, ",")) == NULL)
	 just_show = 1;
      else {
	 PCCOM.baud   = B9600;
	 PCCOM.parity = PNONE;
	 PCCOM.db     = CS8;
	 PCCOM.sb     = ST1;
	 PCCOM.mc     = M_DTR | M_RTS;
	 PCCOM.fc     = F_XONXOFF;
      }

      /*
       * extract other options
       */
      while (s != NULL) {
	 if (strnicmp(s, "38400", 5) == 0)
	    PCCOM.baud = B38400;
	 else if (strnicmp(s, "19200", 5) == 0)
	    PCCOM.baud = B19200;
	 else if (strnicmp(s, "9600", 4) == 0)
	    PCCOM.baud = B9600;
	 else if (strnicmp(s, "4800", 4) == 0)
	    PCCOM.baud = B4800;
	 else if (strnicmp(s, "2400", 4) == 0)
	    PCCOM.baud = B2400;
	 else if (strnicmp(s, "1200", 4) == 0)
	    PCCOM.baud = B1200;
	 else if (strnicmp(s, "600", 3) == 0)
	    PCCOM.baud = B600;
	 else if (strnicmp(s, "300", 3) == 0)
	    PCCOM.baud = B300;
	 else if (stricmp(s, "N") == 0)
	    PCCOM.parity = PNONE;
	 else if (stricmp(s, "O") == 0)
	    PCCOM.parity = PODD;
	 else if (stricmp(s, "E") == 0)
	    PCCOM.parity = PEVEN;
	 else if (stricmp(s, "5") == 0)
	    PCCOM.db = CS5;
	 else if (stricmp(s, "6") == 0)
	    PCCOM.db = CS6;
	 else if (stricmp(s, "7") == 0)
	    PCCOM.db = CS7;
	 else if (stricmp(s, "8") == 0)
	    PCCOM.db = CS8;
	 else if (stricmp(s, "1") == 0)
	    PCCOM.sb = ST1;
	 else if (stricmp(s, "2") == 0)
	    PCCOM.sb = ST2;
	 else if (strnicmp(s, "MC=", 3)==0)  //get modem control value
	    PCCOMGetMC(&PCCOM.mc, s + 3);
	 else if (strnicmp(s, "FC=", 3) == 0)
	    PCCOMGetFC(&PCCOM.fc, s + 3);

	 s = strtok(NULL, ",");
      }

      if (!just_show) {
	 PCCOMSetPort(&PCCOM);
	 PCCOMGetPort(&PCCOM);
      }

      if (verbose || just_show)
	 PCCOMShowPort(&PCCOM);
   } //if
}
