#include <conio.h>
#include <dos.h>

#include "comlib.h"

/*
 *      func :  set communication parameters
 *
 *      input :
 *              portno  : port number   0 .. 64
 *              baud    : baud rate     300, 600, 1200, 2400, 9600, 19200, 38400
 *              db      : data bits     5, 6, 7, 8
 *              parity  : parity bit    0 : nonparity, 1 : odd parity, 3 : even parity
 *              sb      : stop bits     0 : 1 bit, 1 : 2 bits
 *      output :
 *              sts     : port status
 */
int cominit(int portno, unsigned int baud, int db, int parity, int sb)
{
   unsigned char cmd = 0x00;
   union         _REGS reg;

   switch (baud) {
      case B38400 :
         cmd |= 0x20;
         break;
      case B19200 :
         cmd |= 0x00;
         break;
      case  B9600 :
         cmd |= 0xE0;
         break;
      case  B4800 :
         cmd |= 0xC0;
         break;
      case  B2400 :
         cmd |= 0xA0;
         break;
      case  B1200 :
         cmd |= 0x80;
         break;
      case   B600 :
         cmd |= 0x60;
         break;
      case   B300 :
         cmd |= 0x40;
         break;
      default :
         return -1;
   }

   switch (parity) {
      case PNONE :              //non parity
         cmd |= 0x00;
         break;
      case PODD :               //odd parity
         cmd |= 0x04;
         break;
      case PEVEN :              //even parity
         cmd |= 0x0C;
         break;
      default :
         return -1;
   }

   switch (db) {
      case CS5 :
         cmd |= 0x00;
         break;
      case CS6 :
         cmd |= 0x01;
         break;
      case CS7 :
         cmd |= 0x02;
         break;
      case CS8 :
         cmd |= 0x03;
         break;
      default :
         return -1;
   }

   switch (sb) {
      case ST1 :
         cmd |= 0x00;
         break;
      case ST2 :
         cmd |= 0x04;
         break;
      default :
         return -1;
   }

   reg.x.dx = portno;
   reg.h.ah = COMINIT;
   reg.h.al = cmd;
   _int86(0x14, &reg, &reg);

   return reg.x.ax;
}

/*
 *      func : Transmit character and wait
 *
 *      input :
 *              portno  : port number
 *              c       : character transmitted
 *      output :
 *              sts     : port status
 */
int comtxch(int portno, char c)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSEND;
   reg.h.al = c;
   _int86(0x14, &reg, &reg);

   return reg.x.ax;
}

/*
 *      func : Get received character with wait
 *
 *      input :
 *              portno  : port number
 *      output :
 *              c       : input character
 *              sts     : port status
 */
int comrxch(int portno, char *c)
{
  union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMRECV;
   _int86(0x14, &reg, &reg);
   *c = reg.h.al;

   return reg.h.ah;
}

/*
 *      func : return serial port status
 *
 *      input :
 *              portno  : port number
 *      output :
 *              sts     : port status
 */
int comstat(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSTAT;
   _int86(0x14, &reg, &reg);

   return reg.x.ax;
}

int comactive(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMACTIVE;
   _int86(0x14, &reg, &reg);

   return reg.x.ax == 0x1954;
}

void comsetDTR(int portno, int dtr)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETDTR;
   reg.h.al = dtr;
   _int86(0x14, &reg, &reg);
}

void compurgerxbuf(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMPURGERXBUF;
   _int86(0x14, &reg, &reg);
}

int compeek(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMPEEK;
   _int86(0x14, &reg, &reg);

   return reg.x.ax;
}

void comsetflow(int portno, int flow)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETFLOW;
   reg.h.al = flow;
   _int86(0x14, &reg, &reg);
}

int comrxblk(int portno, void *ptr, int size)
{
   union _REGS   reg;
   struct _SREGS sreg;

   reg.x.dx = portno;
   reg.h.ah = COMRXBLK;
   reg.x.cx = size;
#if defined (M_I86CM) || defined (M_I86LM) || defined (M_I86HM)
   reg.x.di = FP_OFF(ptr);
   sreg.es = FP_SEG(ptr);
#else
   reg.x.di = (int)ptr;
   _segread(&sreg);
   sreg.es = sreg.ds;
#endif
   _int86x(0x14, &reg, &reg, &sreg);

   return reg.x.ax;
}

int comtxblk(int portno, void *ptr, int size)
{
   union _REGS   reg;
   struct _SREGS sreg;

   reg.x.dx = portno;
   reg.h.ah = COMTXBLK;
   reg.x.cx = size;

#if defined (M_I86CM) || defined (M_I86LM) || defined (M_I86HM)
   reg.x.di = FP_OFF(ptr);
   sreg.es = FP_SEG(ptr);
#else
   reg.x.di = (int)ptr;
   _segread(&sreg);
   sreg.es = sreg.ds;
#endif
   _int86x(0x14, &reg, &reg, &sreg);

   return reg.x.ax;
}

void combreak(int portno, int brk)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETBREAK;
   reg.h.al = brk;
   _int86(0x14, &reg, &reg);
}

int cominstalled(int portno)
{
   union _REGS                reg;
   struct _SREGS              sreg;
   static struct FOSSIL_INFO info;
   void *ptr = &info;

   info.majver = 0;
   reg.x.dx = portno;
   reg.h.ah = COMINFO;
   reg.x.cx = sizeof(info);
#if defined (M_I86CM) || defined (M_I86LM) || defined (M_I86HM)
   reg.x.di = FP_OFF(ptr);
   sreg.es = FP_SEG(ptr);
#else
   reg.x.di = (int)&info;
   _segread(&sreg);
   sreg.es = sreg.ds;
#endif
   _int86x(0x14, &reg, &reg, &sreg);

   return info.majver >= 5;
}

int comtxs(int portno, char *s)
{
   int sts = 0;

   while (*s) sts |= comtxch (portno, *s++);

   return sts;
}

int comrxs(int portno, char *s)
{
   int t;

   s--;
   do {
      t = comrxch (portno, ++s);
   } while (t == 0 && *s != 13);
   *s = 0;

   return t;
}

int combufstat(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMBUFSTAT;
   _int86(0x14, &reg, &reg);

   return reg.x.cx;
}

void comsetport(int portno, int addr)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETPORT;
   reg.x.bx = addr;

   _int86(0x14, &reg, &reg);
}

void comsetvector(int portno, int addr)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETVECTOR;
   reg.x.bx = addr;
   _int86(0x14, &reg, &reg);
}

void comsetactive(int portno, char active)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETACTIVE;
   reg.h.al = active;
   _int86(0x14, &reg, &reg);
}

void comsetMCR(int portno, int mcr)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETMCR;
   reg.h.al = mcr;
   _int86(0x14, &reg, &reg);
}

char comgetparam(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETPARAM;
   _int86(0x14, &reg, &reg);

   return reg.h.al;
}

void comgetinfo(int portno, unsigned int *baud, int *db, int *parity, int *sb)
{
   unsigned char r, n;
   const unsigned int b[8] = {
      B19200, B38400, B300, B600, B1200, B2400, B4800, B9600
   };
   const int d[4] = {
      CS5, CS6, CS7, CS8
   };
   const int s[2] = {
      ST1, ST2
   };
   const int p[4] = {
      PNONE, PODD, PNONE, PEVEN
   };

   r = comgetparam(portno);
   n = (r >> 5) & 7;
   *baud = b[n];
   n = (r >> 3) & 3;
   *parity = p[n];
   n = (r >> 2) & 1;
   *sb = s[n];
   n = r & 3;
   *db = d[n];
}

char comgetMCR(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETMCR;
   _int86(0x14, &reg, &reg);

   return reg.h.al;
}

char comgetFC(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETFC;
   _int86(0x14, &reg, &reg);

   return reg.h.al;
}

int comgetsets(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETSETS;
   _int86(0x14, &reg, &reg);

   return reg.x.ax;
}

BiosComSet_t *comgetsetinfo(int portno)
{
   union _REGS          reg;
   struct _SREGS        sreg;
   static BiosComSet_t info;
   void *ptr = &info;

   reg.x.dx = portno;
   reg.h.ah = COMGETSETINFO;
#if defined (M_I86CM) || defined (M_I86LM) || defined (M_I86HM)
   reg.x.di = FP_OFF(ptr);
   sreg.es = FP_SEG(ptr);
#else
   reg.x.di = (int)&info;
   _segread(&sreg);
   sreg.es = sreg.ds;
#endif
   _int86x(0x14, &reg, &reg, &sreg);

   return &info;
}

int comgetports(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETPORTS;
   _int86(0x14, &reg, &reg);

   return reg.x.ax;
}

int comdetectport(int portno)
{
   union _REGS           reg;

   reg.x.dx = portno;
   reg.h.ah = COMDETECTPORT;
   _int86(0x14, &reg, &reg);

   return reg.x.ax == 0x55AA;
}

BiosComPort_t *comgetportinfo(int portno)
{
   union _REGS           reg;
   struct _SREGS         sreg;
   static BiosComPort_t info;
   void *ptr;

   reg.x.dx = portno;
   reg.h.ah = COMGETPORTINFO;

#if defined (M_I86CM) || defined (M_I86LM) || defined (M_I86HM)
   reg.x.di = FP_OFF(ptr);
   sreg.es = FP_SEG(ptr);
#else
   reg.x.di = (int)&info;
   _segread(&sreg);
   sreg.es = sreg.ds;
#endif
   _int86x(0x14, &reg, &reg, &sreg);

   return &info;
}

void comrxoff(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMRXOFF;
   _int86(0x14, &reg, &reg);
}

void comrxon(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMRXON;
   _int86(0x14, &reg, &reg);
}

void comsettimeout(int portno, unsigned int timeout)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMSETTIMEOUT;
   reg.x.cx = timeout;
   _int86(0x14, &reg, &reg);
}

unsigned int comgettimeout(int portno)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETTIMEOUT;
   _int86(0x14, &reg, &reg);

   return reg.x.cx;
}

int comsetbaud(int portno, unsigned long baud)
{
    union _REGS reg;

    reg.x.dx = portno;
    reg.x.bx = baud >> 16;
    reg.x.cx = (unsigned int) baud;
    reg.h.ah = COMSETBAUD;
    _int86(0x14, &reg, &reg);

    return reg.x.ax;
}

int comgetbaud(int portno, unsigned long *baud)
{
   union _REGS reg;

   reg.x.dx = portno;
   reg.h.ah = COMGETBAUD;
   _int86(0x14, &reg, &reg);
   *baud = reg.x.bx;
   *baud = (*baud << 16) + reg.x.cx;

   return reg.x.ax;
}


